; -*-Emacs-Lisp-*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; File:         cpp.el
; RCS:          $Header: $
; Description:  C preprocessor utilities
; Author:       Darryl Okahata
; Created:      Sun May 14 22:41:12 1989
; Modified:     Sun May 14 23:45:45 1989 (Darryl Okahata)
; Language:     Emacs-Lisp
; Package:      N/A
;
; (C) Copyright 1989, 1990, 1991 Hewlett-Packard, all rights reserved.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(defvar cpp-program "/lib/cpp"
  "This is the name of the C preprocessor to run.  Some people may
want to use a different C preprocessor, such as the GNU cpp."
)

(defvar cpp-buf "*cpp-buf*"
  "This is the name of the buffer into which the cpp output is placed."
)

(defvar cpp-temp-template "/tmp/cppgnu"
  "The template to use when making a temporary file name for use with
`cpp'.  This is a prefix passed to make-temp-name."
)

(defvar cpp-options nil
  "This string contains command-line options to pass to the C
preprocessor.  It currently exists to allow the user to #define or
#undefine names.  If there are no options to pass to the C preprocessor,
this variable must be nil, not an empty string."
)


(defun cpp (begin end)
  "Run the C preprocessor on the region between the point and mark.
The output is placed in the buffer defined by `cpp-buf'.  The variable
`cpp-options' contains any command-line options to pass to the C
preprocessor."
  (interactive "r")
  (let ( (filename (make-temp-name cpp-temp-template))
       )
    (write-region begin end filename)
    (pop-to-buffer cpp-buf)
    (widen)
    (delete-region (point-min) (point-max))
    (if cpp-options
      (call-process cpp-program filename cpp-buf nil cpp-options)
      (call-process cpp-program filename cpp-buf nil)
    )
    (delete-file filename)
  )
)


(defun cpp-buffer ()
  "Run the current buffer through the C preprocessor."
  (interactive "")
  (cpp (point-min) (point-max))
)


(defun cpp-to-point ()
  "Run the region between `point-min' to `point' through the C
preprocessor."
  (interactive "")
  (cpp (point-min) (point))
)


(defun delete-duplicate-blank-lines-in-region (begin end)
  "In the region, compress successive blank lines into one blank line."
  (interactive "r")
  (let ()
    (narrow-to-region begin end)
    (goto-char (point-min))
    (while (and (re-search-forward "^$" (point-max) t)
		(not (equal (point) (point-max))))
      (delete-blank-lines)
      (next-line 1)
    )
    (widen)
  )
)


(defun cpp-buffer-for-display ()
  "Run the current buffer through the C preprocessor and make it
suitable for viewing.  All occurrences of successive blank lines are each
compressed into one blank line."
  (interactive "")
  (let ()
    (cpp-buffer)
    (delete-duplicate-blank-lines-in-region (point-min) (point-max))
    (goto-char (point-min))
  )
)
